#include <stdio.h>
#include <stdlib.h>
#include <glib-1.2/glib.h>
#include <glib-1.2/gmodule.h>
#include <gtk/gtk.h>

#include "sayhello.h"

/* -------------------------------------------------------------- 

  program (part of the shared_lib sample)
  
  Part of the SourceLib Project at LinuxProgramming.com
  http://www.linuxprogramming.com/SourceLib

  This sample program has been released into the public domain.

  Purpose: Build and use a shared library that contains a single 
  function, sayHello().  This is really just a very fancy version
  of a "hello, world" program.

  Use the command "make" to build this sample, and "make run" 
  to run a simple test.  You can also use "make clean" to delete
  the binaries and links created by the build process.

  See the comments in the file "makefile" for details about how 
  this sample really works.

  ---------------------------------------------------------------

  This is the source for the main program, which calls our sayHello()
  function in the library.  Notice that it looks exactly like
  and standard C source file for calling a separately compiled 
  module.  

  --------------------------------------------------------------- */

/* Change Log:
 *
 * 14 June 02 -  Modified program to load GLIB modules via the g_module
 * API
 *
 * 16 June 02 - Modified program to create a submenu using plug-in's API;
 */

GtkWidget *CreateMenuItem (GtkWidget *menu, 
                           char *szName, 
                           char *szAccel,
                           char *szTip, 
                           GtkSignalFunc func,
                           gpointer data);
GtkWidget *CreateBarSubMenu (GtkWidget *menu, char *szName);
GtkWidget *CreateSubMenu (GtkWidget *menubar, char *szName);


extern GtkWidget *menubar;

static GtkWidget *plugin_menu;
static GtkWidget *libsubmenu;
static GtkWidget *hello_menu;
static GtkWidget *goodbye_menu;

static GModule *hello_module;


int load_plugin ( GtkWidget *widget, gpointer data )
{
  mod_func *f_ptr;

  char *path;
  char *shlib = data;
  char *shlib_path = g_get_current_dir();
  char *shlib_init = "sayHello_init";

  plugin_callbacks *callbacks;


  printf("program is about to call a function in the shared library...\n\n");

  /* --- Check if platform supports use of shared libraries --- */
  if ( g_module_supported() == FALSE )
  {
    fprintf(stderr,"Platform does not support loading of shared libraries\n");
    return(EXIT_FAILURE);
  }

  /* --- build path and name of shared library to load --- */
  path = g_module_build_path(shlib_path, shlib);
  g_return_val_if_fail(path != NULL, -1);

  /* --- load (open) the module for use by program --- */
  hello_module = (GModule *) g_module_open(path, G_MODULE_BIND_LAZY);
  g_return_val_if_fail(hello_module != NULL, -1);
  
  /* --- Find function symbol(s) --- */
  if (g_module_symbol(hello_module, shlib_init, (gpointer) &f_ptr) == FALSE)
  {
    fprintf(stderr, "File: %s, Line: %d - Could not find function name [%s] on plug-in %s.\n", __FILE__, __LINE__, "sayHello", shlib);
    return (EXIT_FAILURE);
  }

  /* --- call funtion inside freshly loaded plug-in --- */
  printf("finished loading shared libray, calling plug-in function ...\n\n");
  callbacks = f_ptr("Retrieving API from plug-in module:");

  plugin_menu = CreateBarSubMenu ( menubar, "plug-ins" );

  libsubmenu = CreateSubMenu( plugin_menu, shlib );
  
  hello_menu = CreateMenuItem (libsubmenu, 
			       callbacks[0].menu_action_name, 
			       callbacks[0].shortcut, 
			       callbacks[0].tooltip, 
			       GTK_SIGNAL_FUNC (callbacks[0].mod_func), 
			       callbacks[0].data);

  goodbye_menu = CreateMenuItem (libsubmenu, 
				 callbacks[1].menu_action_name, 
				 callbacks[1].shortcut, 
				 callbacks[1].tooltip, 
				 GTK_SIGNAL_FUNC (callbacks[1].mod_func), 
				 callbacks[1].data);


  /* --- finished with the plug-in, now unload it --- */
/*    printf("\n...back in program. Now unload the plug-in. \n\n"); */
/*    if ( !g_module_close (hello_module) ) */
/*      fprintf(stderr,"ERROR: %s\n", g_module_error() ); */

  /* Indicate normal termination via the EXIT_SUCCESS constant from stdlib.h */
  return EXIT_SUCCESS;
}


int unload_plugin ( GtkWidget *widget, gpointer data )
{
  char *mod;

  gtk_menu_item_remove_submenu( GTK_MENU_ITEM(hello_menu) );
  gtk_menu_item_remove_submenu( GTK_MENU_ITEM(goodbye_menu) );

  gtk_widget_hide( hello_menu );
  gtk_widget_destroy( hello_menu );
  gtk_widget_hide( goodbye_menu );
  gtk_widget_destroy( goodbye_menu );

  gtk_menu_item_remove_submenu( GTK_MENU_ITEM(libsubmenu) );
  gtk_widget_hide( libsubmenu );
  gtk_widget_destroy( libsubmenu );

  mod = g_module_name( hello_module );
  printf("the module name to close: %s\n", mod);

  /* --- finished with the plug-in, now unload it --- */
  printf("\n...back in program. Now unload the plug-in. \n\n");
  if ( !g_module_close ( (GModule *)hello_module ))
    fprintf(stderr,"ERROR: %s\n", g_module_error() );

  /* Indicate normal termination via the EXIT_SUCCESS constant from stdlib.h */
  return EXIT_SUCCESS;
}
